"""
check for cyclic dependencies
"""
from textx import get_children_of_type
from textx import get_location
from textx import textx_isinstance


class CyclicDependencyError(RuntimeError):
    """raise this exception if a cyclic dependency is detected"""
    def __init__(self, var, ref):
        msgs = [f'variable: {var.name}, loc: {repr(get_location(var))}']
        msgs.append(f'reference: {repr(get_location(ref))}')
        message = 'cyclic dependency detected:\n'+'\n'.join(msgs)
        super().__init__(message)


def detect_cycles(lhs_var, rhs_var, meta):
    """recursively walk through the dependency tree"""
    rhs_vars = set()
    for var_ref in get_children_of_type('VarReference', rhs_var.parameter):
        var = var_ref.ref
        if textx_isinstance(var, meta['Variable']):
            rhs_vars.add(var)
            if var is not lhs_var:
                rhs_vars.update(detect_cycles(lhs_var, var, meta))
            else:
                raise CyclicDependencyError(lhs_var, var_ref)
    return rhs_vars


def check_cycles_processor(model, metamodel):
    """model processor to check for cyclic dependencies in the program"""
    for var in get_children_of_type('Variable', model):
        detect_cycles(var, var, metamodel)
