"""
check for duplicated initializations
"""
from textx import get_children_of_type
from textx import get_location


class InitializationError(Exception):
    """raise this exception if an object is defined repeatedly"""
    def __init__(self, dobjs):
        msgs = [f'obj: {v.name}, loc: {repr(get_location(v))}' for v in dobjs]
        message = 'repeated initialization:\n'+'\n'.join(msgs)
        super().__init__(message)


def check_duplicates_processor(model, metamodel):
    """
    processor to detect all duplicate object initializations
    """
    names = ['Variable', 'FunctionDefinition', 'ObjectImport']
    objs = []
    for name in names:
        objs.extend(get_children_of_type(name, model))
    seen = set()
    dobjs = [v for v in objs if v.name in seen or seen.add(v.name)]
    try:
        assert len(dobjs) == 0
    except AssertionError as err:
        raise InitializationError(dobjs) from err
