"""
contraints applied to imported objects
"""
from textx import get_children_of_type
from textx import get_location
from textx import get_metamodel
from textx import textx_isinstance


class ObjectImportError(ImportError):
    """raise this exception if an object cannot be imported"""
    def __init__(self, obj):
        obj_namespace = '.'.join(obj.namespace)
        msg = f'obj: {obj_namespace}.{obj.name}, loc: {repr(get_location(obj))}'
        message = 'object could not be imported:\n' + msg
        super().__init__(message)


def get_object_import(obj):
    """return an imported object"""
    assert hasattr(obj, 'name') and hasattr(obj, 'namespace')
    try:
        module = __import__('.'.join(obj.namespace), fromlist=[obj.name], level=0)
    except ImportError:
        raise ObjectImportError(obj)
    try:
        retval = getattr(module, obj.name)
    except AttributeError:
        raise ObjectImportError(obj)
    return retval


def check_imports_processor(model, metamodel):
    """check all imported objects in a program"""
    for obj in get_children_of_type('ObjectImport', model):
        get_object_import(obj)


def check_function_import(obj):
    """check that an imported object is callable"""
    assert textx_isinstance(obj, get_metamodel(obj)['ObjectImport'])
    get_object_import(obj)
    try:
        assert callable(get_object_import(obj))
    except AssertionError:
        msg = f'obj: {obj.namespace}.{obj.name}, loc: {repr(get_location(obj))}'
        message = 'imported object is not callable:\n' + msg
        raise TypeError(message)
