"""
Model processors are callables that are called at the end of the parsing when
the whole model is instantiated. These processors accept the model and metamodel
as parameters.

Object processors are registered for particular classes (grammar rules) and are
called when the objects of the given class is instantiated. Object processors
are registered by defining a map between a rule name and the callable that will
process the instances of the metamodel class correspoding to the rule. The rule
must be a match rule and no abstract rule.
"""
from cyclic import check_cycles_processor
from duplicates import check_duplicates_processor
from functions import check_functions_processor
from imports import check_imports_processor
from typechecks import check_types_processor


def add_processors(metamodel):
    """register the processors on the metamodel instance"""
    metamodel.register_model_processor(check_duplicates_processor)
    metamodel.register_model_processor(check_cycles_processor)
    metamodel.register_model_processor(check_imports_processor)
    metamodel.register_model_processor(check_functions_processor)
    metamodel.register_model_processor(check_types_processor)
