"""deepcopy implementation for metamodel classes"""
from copy import deepcopy


def __deepcopy__(self, memo):
    cls = self.__class__
    result = cls.__new__(cls)
    memo[id(self)] = result
    for key, val in self.__dict__.items():
        if key == 'parent':
            # must stay a reference
            result.parent = self.parent
        elif key == 'func':
            result.func = self.func
        else:
            # must be copied
            setattr(result, key, deepcopy(val, memo))
    self.last_copy = result
    return result


def add_deepcopy(metamodel):
    """Add deepcopy method to metamodel classes using monkey style patching"""
    metamodel['IfExpression'].__deepcopy__ = __deepcopy__
    metamodel['IfFunction'].__deepcopy__ = __deepcopy__
    metamodel['FunctionCall'].__deepcopy__ = __deepcopy__
    metamodel['Variable'].__deepcopy__ = __deepcopy__
    metamodel['Dummy'].__deepcopy__ = __deepcopy__
    metamodel['VarReference'].__deepcopy__ = __deepcopy__
    metamodel['Factor'].__deepcopy__ = __deepcopy__
    metamodel['Term'].__deepcopy__ = __deepcopy__
    metamodel['Expression'].__deepcopy__ = __deepcopy__
    metamodel['Operand'].__deepcopy__ = __deepcopy__
    metamodel['BooleanOperand'].__deepcopy__ = __deepcopy__
    metamodel['And'].__deepcopy__ = __deepcopy__
    metamodel['Or'].__deepcopy__ = __deepcopy__
    metamodel['Not'].__deepcopy__ = __deepcopy__
    metamodel['Equal'].__deepcopy__ = __deepcopy__
    metamodel['LessThan'].__deepcopy__ = __deepcopy__
    metamodel['GreaterThan'].__deepcopy__ = __deepcopy__
    metamodel['LessThanOrEqual'].__deepcopy__ = __deepcopy__
    metamodel['GreaterThanOrEqual'].__deepcopy__ = __deepcopy__
    metamodel['Tuple'].__deepcopy__ = __deepcopy__
