# pylint: disable=protected-access
"""properties and attributes of FunctionCall class"""
from copy import deepcopy
from textx import get_children_of_type, get_children, get_parent_of_type
from textx import get_metamodel
from textx import textx_isinstance

comparisons = ['Equal', 'LessThan', 'GreaterThan', 'LessThanOrEqual',
               'GreaterThanOrEqual']


def complete_expr(self):
    """substitute all references to dummy variables with call parameters"""
    meta = get_metamodel(self)
    vrefs = get_children_of_type('VarReference', self.__expr)
    for vref in filter(lambda x: textx_isinstance(x.ref, meta['Dummy']), vrefs):
        if hasattr(vref.ref, 'name'):
            for par, arg in zip(self.params, self.func.args):
                if vref.ref.name == arg.name:
                    if textx_isinstance(par, meta['VarReference']):
                        vref.ref = par.ref
                    else:
                        vref.ref = par
                    break
    return self.__expr


def add_function_call_properties(metamodel):
    """Add complete_expr() method using monkey style patching"""
    metamodel['FunctionCall'].expr = property(complete_expr)
    metamodel['FunctionCall'].complete_expr = complete_expr


def function_call_processor(model, metamodel):
    """copy expression tree from the function definition object"""
    if not get_parent_of_type('FunctionDefinition', model):
        for call in get_children_of_type('FunctionCall', model):
            if not call.__expr:
                if textx_isinstance(call.func, metamodel['FunctionDefinition']):
                    call.__expr = deepcopy(call.func.expr)
                    new_objs = get_children(lambda x: True, call.__expr)
                    for new_obj in new_objs:
                        if new_obj is not call.__expr:
                            new_obj.parent = new_obj.parent.last_copy
                    call.__expr.parent = call
                    function_call_processor(call.expr, metamodel)
                else:
                    call.__expr = None
