# pylint: disable=protected-access
"""define loader methods"""
import pandas
from textx import get_metamodel
from interpreter import ioops


def load_plain_type(self):
    """load plain type object from a file or URL"""
    assert self.need_input
    self.__value = ioops.load_value(self.url, self.filename)
    assert isinstance(self.__value, self.type_)
    self.need_input = False


def load_series(self):
    """load a series object from a file or URL"""
    assert self.need_input
    val = ioops.load_value(self.url, self.filename)
    assert isinstance(val, dict)
    self.name = val['name']
    self.elements = val['data']
    self.need_input = False
    self.__value = pandas.Series(**val)
    self.__type = (type(self.__value[0].item()), len(self.elements))


def load_table(self):
    """load a table object from a file or URL"""
    assert self.need_input
    val = ioops.load_value(self.url, self.filename)
    assert isinstance(val, dict)
    self.__value = pandas.DataFrame.from_dict(val, orient='columns')
    series = list(self.__value.to_dict(orient='series').values())
    types = tuple(type(s[0].item()) for s in series)
    self.__type = (types, len(self.__value))
    self.columns = []
    for ser, typ in zip(series, types):
        ser_obj = get_metamodel(self)['Series']()
        setattr(ser_obj, 'elements', ser.to_list())
        setattr(ser_obj, 'name', ser.name)
        setattr(ser_obj, 'parent', self)
        setattr(ser_obj, 'need_input', False)
        setattr(ser_obj, 'url', None)
        setattr(ser_obj, 'filename', None)
        # setattr(ser_obj, 'value', ser)
        # setattr(ser_obj, 'type_', typ)
        self.columns.append(ser_obj)
    self.need_input = False


def add_object_loaders(metamodel):
    """add loader methods to metamodel classes"""
    metamodel['String'].load = load_plain_type
    metamodel['Number'].load = load_plain_type
    metamodel['Bool'].load = load_plain_type
    metamodel['Series'].load = load_series
    metamodel['Table'].load = load_table
