"""
tests for recursive functions
"""
import os
import pytest
from textx import metamodel_from_file
from textx import get_children_of_type
from textx import textx_isinstance
from constraints import check_function_definition, check_function_call
from metamodel import add_properties


@pytest.fixture(name='metamodel')
def fixture_metamodel():
    """parse the grammar and generate the object classes"""
    this_folder = os.path.abspath(os.path.dirname(__file__))
    mm_file = os.path.join(this_folder, '../grammar/virtmat.tx')
    mm_obj = metamodel_from_file(mm_file, auto_init_attributes=False)
    add_properties(mm_obj)
    return mm_obj


def test_function_call_if_function_recursion(metamodel):
    """test function call with if-function recursion"""
    prog_inp = 'gt(x, y) = if(x>y, gt(x, y), x)\n'
    prog = metamodel.model_from_str(prog_inp)
    funcs = get_children_of_type('FunctionDefinition', prog)
    calls = get_children_of_type('FunctionCall', prog)
    assert len(funcs) == 1
    with pytest.raises(AssertionError):
        check_function_definition(funcs[0], metamodel)
    assert textx_isinstance(calls[0].parent, metamodel['IfFunction'])
    assert len(calls[0].params) == len(funcs[0].args)
    assert len(calls[0].params) == 2
    check_function_call(calls[0], metamodel)


def test_recursive_function_call_two_arguments(metamodel):
    """test recursive function call with two arguments"""
    prog_inp = 'a = pow(2, 3); pow(x, n) = if(n>0, x*pow(x, n-1), 1)\n'
    prog = metamodel.model_from_str(prog_inp)
    func_defs = get_children_of_type('FunctionDefinition', prog)
    assert len(func_defs) == 1
    with pytest.raises(AssertionError):
        check_function_definition(func_defs, metamodel)
    calls = get_children_of_type('FunctionCall', prog)
    for call in calls:
        check_function_call(call, metamodel)
        assert (textx_isinstance(call.parent, metamodel['Variable']) or
                textx_isinstance(call.parent, metamodel['Operand']))
