""" run a workflow using fireworks """
import sys
from textx import metamodel_from_file
from textx import get_children_of_type
from textx import textx_isinstance
from fireworks import PyTask, Firework, Workflow, ScriptTask
from fireworks.core.launchpad import LaunchPad
from fireworks.utilities.dagflow import DAGFlow
from fireworks.fw_config import LAUNCHPAD_LOC
from wfquery import WFQuery

metamodel_file = 'workflow.tx'
model_file = sys.argv[1]

wfmeta = metamodel_from_file(metamodel_file)
wf = wfmeta.model_from_file(model_file)
launchpad = LaunchPad.from_file(LAUNCHPAD_LOC)
fireworks = []
for index, node in enumerate(wf.nodes):
    spec = {}
    node.id = index
    tasks = []
    for t in node.tasks:
        Pytask = wfmeta['Pytask']
        Script = wfmeta['Script']
        metaclass_pytask = textx_isinstance(t, Pytask)
        metaclass_script = textx_isinstance(t, Script)
        if metaclass_pytask is True:
            data = {d.name: d.value for i in t.inputs
                    for d in get_children_of_type('Data', i)}
            spec.update(data)
            tasks.append(PyTask(func=t.func, inputs=[i.name for i in t.inputs],
                                outputs=[i.name for i in t.outputs]))
        else:
            tasks.append(ScriptTask(script=t.script, stdout_file=t.stdout_file,
                                    stdin_file=t.stdin_file,
                                    stderr_file=t.stderr_file))
    for t in node.files_in:
        data = {d.name: d.value for d in get_children_of_type('Filesin', t)}
        filein = {"_files_in": data}
        spec.update(filein)

    for t in node.files_out:
        data = {d.name: d.value for d in get_children_of_type('Filesout', t)}
        fileout = {"_files_out": data}
        spec.update(fileout)
    fireworks.append(Firework(tasks, spec=spec, name=node.name))

links = {fireworks[i]: [] for i, n in enumerate(wf.nodes)}
for node in wf.nodes:
    for parent in node.parents:
        links[fireworks[parent.id]].append(fireworks[node.id])

fw_wf = Workflow(fireworks, links, name=wf.name)
DAGFlow.from_fireworks(fw_wf).check()
print(fw_wf.to_dict())
launchpad.add_wf(fw_wf)

query=[]        
for index, que in enumerate(wf.queries):
    spec = {}
    que.id = index
    for t in que.nodes:
        if all([isinstance(item, int) for item in t.nodedetail]):
            query.append({'name': {'$in' : t.nodedetail}})
        elif all([isinstance(item, str) for item in t.nodedetail]):
            query.append({'nodes': {'$in' : t.nodedetail}})
        else:
            print("Ivalid entry")

wfq = WFQuery(launchpad, wf_query=query[0])
